#!/usr/bin/env python
# -*- coding: utf-8 -*-

# # 2A.ml - Classification binaire avec features textuelles - correction
# 
# Ce notebook propose de voir comment incorporer des features pour voir l'amélioration des performances sur une classification binaire. 

# In[1]:


from jyquickhelper import add_notebook_menu
add_notebook_menu()


# In[2]:


get_ipython().run_line_magic('matplotlib', 'inline')


# ## Récupérer les données
# 
# Les données sont téléchargeables [Compétition 2017 - additifs alimentaires](http://www.xavierdupre.fr/app/ensae_teaching_cs/helpsphinx/questions/competition_2A.html#id1) ou encore avec le code :

# In[3]:


from pyensae.datasource import download_data
data_train = download_data("off_train_all.zip", 
                           url="https://raw.githubusercontent.com/sdpython/data/master/OpenFoodFacts/")


# In[4]:


data_test = download_data("off_test_all.zip", 
                           url="https://raw.githubusercontent.com/sdpython/data/master/OpenFoodFacts/")


# In[5]:


import pandas
df = pandas.read_csv("off_test_all.txt", sep="\t", encoding="utf8", low_memory=False)


# In[6]:


df.head()


# ## Exercice 1 : extraire un échantillon aléatoire
# 
# Les données sont volumineuses. Prenons un échantillon pour aller plus vite. On se sert de la méthode [sample](https://pandas.pydata.org/pandas-docs/stable/generated/pandas.DataFrame.sample.html).

# In[7]:


sample = df.sample(n=10000)
sample.shape


# ## Exercice 2 : caler un premier modèle de classification binaire
# 
# Avec les variables numériques uniquement. La cible est la variable ``hasE``. On sélectionne les variables numériques et on cale un premier modèle et la variable ``hasE`` dont le type est booléen. On remplace les valeurs manquantes par zéro.

# In[8]:


sample_num = sample.select_dtypes((float, bool))
sample_num.shape


# In[9]:


X = sample_num.drop("hasE", axis=1).fillna(0)
Y = sample_num["hasE"]


# In[10]:


from sklearn.linear_model import LogisticRegression
clf = LogisticRegression()
clf.fit(X, Y)


# On vérifie que le modèle fonctionne à peu près.

# In[11]:


from sklearn.metrics import confusion_matrix
conf = confusion_matrix(Y, clf.predict(X))
conf


# Ca ne marche pas vraiment. On essaye avec une forêt d'arbres.

# In[12]:


from sklearn.ensemble import RandomForestClassifier
clrf = RandomForestClassifier()
clrf.fit(X, Y)


# In[13]:


conf = confusion_matrix(Y, clrf.predict(X))
conf


# Beaucoup mieux. Les performances semblent incroyables mais il ne faut pas oublier qu'on évalue le modèle sur la base d'apprentissage. Essayons sur un autre échantillon aléatoire.

# In[14]:


sample2 = df.sample(n=10000)
sample2_num = sample2.select_dtypes((float, bool))
X2 = sample2_num.drop("hasE", axis=1).fillna(0)
Y2 = sample2_num["hasE"]
conf2 = confusion_matrix(Y2, clrf.predict(X2))
conf2


# Pas si mal.

# ## Exercice 3 : tracer une courbe ROC
# 
# On trace la courbe ROC avec le second échantillon.

# In[15]:


from sklearn.metrics import roc_curve
y_proba = clrf.predict_proba(X2)
y_pred = clrf.predict(X2)
# y_pred est plein de booléen. Il faut les convertir en entier.
prob_pred = y_proba[:, 1]
fpr, tpr, th = roc_curve(Y2, prob_pred)


# In[16]:


import matplotlib.pyplot as plt
plt.figure()
lw = 2
plt.plot(fpr, tpr, color='darkorange', lw=lw, label='Courbe ROC')
plt.plot([0, 1], [0, 1], color='navy', lw=lw, linestyle='--')
plt.xlim([0.0, 1.0])
plt.ylim([0.0, 1.05])
plt.xlabel("Proportion mal classée")
plt.ylabel("Proportion bien classée")
plt.title('ROC')
plt.legend(loc="lower right")


# La courbe [ROC](http://www.xavierdupre.fr/app/mlstatpy/helpsphinx/c_metric/roc.html) paraît fidèle à la matrice de confusion, on vérifie avec la distribution des scores.

# In[17]:


fig, ax = plt.subplots(1, 1)
ax.hist(y_proba[Y2==1, 1], alpha=0.5, label="score class 1")
ax.hist(y_proba[Y2==0, 1], alpha=0.5, label="score class 0")
ax.legend();


# ## Exercice 4 : utiliser la variable qui contient le pays
# 
# Est-ce que le modèle est plus performant ?

# In[18]:





# ## Exercice 5 : utiliser la variable additives_tags ?
# 
# Les résultats ne vous paraissent-ils pas étranges ?

# In[19]:





# ## Exercice 6 : utiliser la variable ingredients_text ?
# 
# Que suggérez vous *embedding* ?

# In[20]:





# ## Exercice 7 : utiliser la variable product_name ?

# In[21]:





# ## Exercice 8 : utiliser GridSearch pour optimiser un hyper-paramètre du modèle ?

# In[22]:





# ## Exercice 9 : faites tourner une cross validation ?

# In[23]: